/**
 * This is the cellular programming algorithm (CPA) of density task very similar
 * to Moshe Sipper's.<br>
 * Cellular automata (CA) is 1 dimensional, has 2 states, and is non-uniform.
 * Each cell has 3 neighbours (one left, itself, and one right). Initial rules
 * are random, CAs are also random. Rules are 8 bits long. Spatially periodic
 * boundary conditions are applied.
 * 
 * @author TineL
 * @version 1.1, 2007-05-30
 * @since 1.0
 */
public class CPA {

  /** Number of cells. */
  private final static int CELL_NUM=19;
  /** Number od execution steps. */
  private final static int STEP_NUM=CELL_NUM+2;
  /** Number of start configurations. */
  private final static int START_CONFIG_NUM=300;
  /** Number of generations. */
  private final static int GENERATION_NUM=200;

  /** Rule mutation probability. */
  private static double MUTATION_PROBABILITY=0.01;

  /**
   * Start method.
   * 
   * @param args the command line arguments (not used).
   * @since 1.0
   */
  public static void main(String[] args) {
    long time=System.currentTimeMillis();

    CPA cpa=new CPA();
    cpa.run();

    long duration=System.currentTimeMillis()-time;
    System.out.println("Run for "+((double)duration/1000)+" s.");
  }

  /**
   * Runs the CPA.
   * 
   * @since 1.0
   */
  public void run() {
    // Generate rule set
    System.out.println("Generating rules...");
    Rules rules=Rules.generateRandomRules(CELL_NUM);
    System.out.println("  "+rules);

    double bestFitnessSuccess=0;

    // Generations
    for (int g=0; g<GENERATION_NUM; g++) {
      System.out.println("Starting new generation #"+(g+1)+"...");

      Fitnesses fitnesses=new Fitnesses(CELL_NUM);

      // ////////////////////////////////////////
      // Rules evaluation
      // ////////////////////////////////////////

      for (int c=0; c<START_CONFIG_NUM; c++) {
        // Generate initial configuration
        double onesRatio=Math.random();
        CA ca=CA.generateRandomCA(CELL_NUM, onesRatio);
        boolean hasMoreOnes=ca.moreOnes();

        // Execution
        for (int m=0; m<STEP_NUM; m++) {
          CA caNext=new CA(CELL_NUM);
          for (int i=0; i<CELL_NUM; i++) {
            caNext.set(i, rules.get(i).traverse(false, false, ca.getLeft(i),
              ca.get(i), ca.getRight(i))); // Using only 8 bits of rule
          }
          ca=caNext;
        }

        // Fitness
        for (int i=0; i<CELL_NUM; i++) {
          if (!hasMoreOnes&&!ca.get(i)) {
            // Cell is 0, CA had > 50 % of 0s
            fitnesses.inc(i);

          } else if (hasMoreOnes&&ca.get(i)) {
            // Cell is 1, CA had >= 50 % of 1s
            fitnesses.inc(i);
          }
        }
      }

      // ////////////////////////////////////////
      // Fitness analyze
      // ////////////////////////////////////////

      int fitnessSum=fitnesses.sum();
      double fitnessSuccess=(double)fitnessSum/(CELL_NUM*START_CONFIG_NUM)*100;

      // Print out analysis if fitness is best in this generation
      if (fitnessSuccess>bestFitnessSuccess) {
        System.out.println("  "+fitnesses);
        System.out.println("  fitnessSuccess="+fitnessSuccess+"%");
        System.out.println("  "+rules);
        bestFitnessSuccess=fitnessSuccess;
      }
      if (bestFitnessSuccess==100.0) break; // End

      // ////////////////////////////////////////
      // Rules evolution
      // ////////////////////////////////////////

      Rules newRules=new Rules(rules);

      for (int i=0; i<CELL_NUM; i++) {
        // Number of fitter (better) neighbour rules for a cell
        int fitters=fitnesses.biggerNeighbours(i);

        if (fitters==0) {
          // Left or right rule is not better

          // Rule is left unchanged

        } else if (fitters==1) {
          // Left or right rule is better

          // Better rule is used
          if (fitnesses.isLeftBigger(i)) {
            newRules.set(i, rules.getLeft(i));
          } else {
            newRules.set(i, rules.getRight(i));
          }

          // Mutate
          newRules.set(i, newRules.get(i).mutate(MUTATION_PROBABILITY));

        } else if (fitters==2) {
          // Left & right rules are better

          // Crossover two better rules
          Rule crossoverRule=new Rule(rules.getRight(i));
          int cutter=(int)(Math.random()*8); // Using only 8 bits of rule
          for (int j=cutter; j<8; j++) {
            crossoverRule.set(j, rules.getLeft(i).get(j));
          }
          newRules.set(i, crossoverRule);

          // Mutate
          newRules.set(i, newRules.get(i).mutate(MUTATION_PROBABILITY));
        }
      }
      rules=newRules;
    }

    // End
    System.out.println("Fitness success is "+bestFitnessSuccess+"%. Finished.");
  }
}
