/**
 * The 1 dimensional grid of rules with some utilities.
 * 
 * @author TineL
 * @version 1.1, 2007-05-30
 * @since 1.0
 */
public class Rules extends Abstract1DGrid {

  /** Array of rules. */
  private Rule[] rules;

  /**
   * Creates <code>Rules</code> with the specified size. All rules are set to
   * 0 code.
   * 
   * @param size the grid size (number of rules).
   * @since 1.0
   */
  public Rules(int size) {
    super(size);
    this.rules=new Rule[size];
  }

  /**
   * Creates <code>Rules</code> from the specified rules.
   * 
   * @param rules the rules (<code>null</code> not permitted).
   * @since 1.1
   */
  public Rules(Rules rules) {
    this(rules.size());
    for (int i=0; i<size(); i++) {
      set(i, rules.get(i));
    }
  }

  /**
   * Generates a grid of random rules.
   * 
   * @param size the grid size (number of rules).
   * @return a grid of random rules (<code>null</code> not possible).
   * @since 1.0
   */
  public static Rules generateRandomRules(int size) {
    Rules rules=new Rules(size);
    for (int i=0; i<size; i++) {
      rules.set(i, Rule.generateRandomRule());
    }
    return rules;
  }

  /**
   * Generates a grid of specific rules defined by the ruleCode. All generated
   * rules are the same.
   * 
   * @param size the grid size (number of rules).
   * @param ruleCode the rule code for all rules.
   * @return a grid of rules (<code>null</code> not possible).
   * @since 1.0
   */
  public static Rules generateRules(int size, int ruleCode) {
    Rules rules=new Rules(size);
    for (int i=0; i<size; i++) {
      rules.set(i, new Rule(ruleCode));
    }
    return rules;
  }

  /**
   * @param index the rule index.
   * @return the rule (<code>null</code> not possible).
   * @since 1.0
   */
  public Rule get(int index) {
    return rules[index];
  }

  /**
   * Replaces the old rule with the given rule at the specified index.
   * 
   * @param index the rule index.
   * @param rule the new rule (<code>null</code> not permitted).
   * @since 1.0
   */
  public void set(int index, Rule rule) {
    rules[index]=rule;
  }

  /**
   * @param index the rule index.
   * @return the rule on the left (neighbour) (<code>null</code> not
   *         possible).
   * @since 1.0
   */
  public Rule getLeft(int index) {
    return get(getLeftIndex(index));
  }

  /**
   * @param index the rule index.
   * @return the rule on the right (neighbour) (<code>null</code> not
   *         possible).
   * @since 1.0
   */
  public Rule getRight(int index) {
    return get(getRightIndex(index));
  }

  /*
   * @since 1.0
   */
  public String toString() {
    String out="Rules=[";
    for (int i=0; i<size()-1; i++) {
      out+=get(i).getCode()+"|";
    }
    out+=get(size()-1).getCode()+"]";
    return out;
  }
}
